/***************************************************************************
 *   Copyright (C) 2011-2013 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GAME_WIDGET_H
#define GAME_WIDGET_H

#include "../global.h"
#include "sprite.h"
#include "level.h"

#include <QGraphicsView>
#include <QMap>
#include <QUrl>

class QGraphicsScene;
class QGraphicsSimpleTextItem;
class QGraphicsRectItem;
class QKeyEvent;
class Sprite;
class QSignalMapper;
class QLabel;
class QPushButton;
class QToolBar;
class QActionGroup;
class SoundEngine;
class ParameterWidget;
class QuickTimer;

//Main class to play a video game
class GameWidget : public QGraphicsView {
	Q_OBJECT
  public:
	explicit GameWidget( QWidget *parent=0 );
	
	void setLevel(const QString& levelFileName);
	QStringList filesDependencyList(bool isCreditsDependencies);
	
	//sound engine to play and mix sounds from wav files
	//this is only a copy of the pointer from flukzwidget (bad practice to avoid creating signals)
	SoundEngine *soundEngine;
	
	//public for access from fluzwidget (not good design)
	ParameterWidget* parameterWidget;
	
  public slots:
	//game mode
	void startGame();
	void trigGameTest();
	void stopGame();
	
	//edit mode
	void startEdit();
	void closeEditMode();
	void saveTemporaryFile();
	
  signals:
	void sceneChanged();
	void askPage(const QString&);
	void gameTestFinished();
	
  private:
	QGraphicsScene* menuScene;	//different scene that is used as a menu
	QGraphicsScene* gameScene;	//main scene that displays the sprites
	QGraphicsScene* editScene;	//scene that displays the entire level for edition
	
	Level* level;					//level object with the positions of all sprites
	QString currentLevelName;
	QString expectedNextLevelName;	//remember the name of the next level to continue with the same score
	
	Sprite* playerSprite;					//player main sprite (control with keyboard)
	QGraphicsSimpleTextItem * scoreItem;	//item to display the score
	int levelScore;
	int previousLevelScore;					//score at the end of the previous level
	QGraphicsRectItem * energyBarItem;
	QGraphicsRectItem * energyBarGrayItem;
	int energyBarFullPoints;
	int energyBarCurrentPoints;
	
	//QuickTimer is used instead of QTimer because of high cpu usage of QTimer on some platforms
	QuickTimer* gameTimer;		//timer to animate the game
	QMap<int, int> keyMap;		//map of the keys (in order to customize the keys)
	
	enum gameResult {GameOver=0, GameRunning=1, LevelComplete=2};
	QLabel *labelResult, *labelScore;
	QPushButton *quitButton, *playButton;
	
	void setBackgroundColor(const std::string& color);
	void createGameScene();
	void createEditScene();
	void createMenuScene();
	
	int gameStatus;
	
	QString cursorPixmapName;
	
	//used in edit mode
	void setEditSceneLimits();
	void setMouseShape(QString shortPixmapName);
	void createSpriteBar();
	void addSpriteAction(QString iconName, QString description, int spriteIndex, bool isChecked);
	void changeActionIcon(QString iconName, int spriteIndex);
	QToolBar* spriteBar;
	QActionGroup* spriteBarGroup;
	QSignalMapper *spriteBarMapper;		//connexion mapping for sprite selection in the toolbar
	int barSelectedSprite;
	bool addSpriteMode;
	Sprite * currentSelectedSprite;
	bool isEditMode, isTestMode;
	
	enum parametersMode {SpriteParameterMode=0, LevelParameterMode=1, SpriteKindParameterMode=2};
	int parameterWidgetMode;			//used to remember the parameters currently displayed (spriteparameter, levelparameter, etc...) 
	
  private slots:
	//game mode
	void deleteAllSprites();
	void createFireSprite(SpriteParameter paramFire);
	void destroySprite();								//delete the sender sprite from the scene
	void readLevel();
	void playSound(int index, int repeatTimes=0);		//play a sound given by its index in the list of sound files
	void displayResult();
	void updateResultText();
	void updateScoreText();
	void playLevel();			//called when the play button is clicked (same level or next level)
	void quitGameWidget();		//called when the quit button is clicked
	
	//score
	void createScoreSprite();
	void setScore(int score);
	void changeScore(int scoreVariation);
	void updateScoreLabel();
	
	//energy bar
	void createEnergyBarItem();
	void setEnergyBarFull(int fullValue);
	void setEnergyBarCurrent(int currentValue);
	
	//editor mode
	void setParameterMode(int mode);
	void spriteBarClicked(int spriteIndex);
	void addSpriteToEditScene(SpriteParameter p);
	void spriteSelected(SpriteParameter p);
	void spriteMoved(SpriteParameter p);
	void parameterChanged(std::string s);
	void removeCurrentSprite();							//remove from the edit scene the currently selected sprite
	void editSelectionChanged();
	
  protected:
 	virtual void mousePressEvent(QMouseEvent *mouseEvent);
	virtual void mouseReleaseEvent(QMouseEvent *mouseEvent);
	virtual void keyPressEvent(QKeyEvent *event);
	virtual void keyReleaseEvent(QKeyEvent *event);
};

#endif
